// Fig. 19.25: PathNavigator.cs
// Demonstrates class XPathNavigator.
using System;
using System.Windows.Forms;
using System.Xml.XPath; // contains XPathNavigator

namespace PathNavigator
{
   public partial class PathNavigatorForm : Form
   {
      public PathNavigatorForm()
      {
         InitializeComponent();
      } // end constructor

      private XPathNavigator xPath; // navigator to traverse document

      // references document for use by XPathNavigator
      private XPathDocument document;

      // references TreeNode list used by TreeView control
      private TreeNode tree;

      // initialize variables and TreeView control
      private void PathNavigatorForm_Load( object sender, EventArgs e )
      {
         // load XML document
         document = new XPathDocument( "sports.xml" ); 
         xPath = document.CreateNavigator(); // create navigator
         tree = new TreeNode(); // create root node for TreeNodes

         tree.Text = xPath.NodeType.ToString(); // #root
         treePath.Nodes.Add( tree ); // add tree

         // update TreeView control
         treePath.ExpandAll(); // expand tree node in TreeView
         treePath.Refresh(); // force TreeView update
         treePath.SelectedNode = tree; // highlight root
      } // end method PathNavigatorForm_Load

      // process btnSelect_Click event
      private void btnSelect_Click( object sender, EventArgs e )
      {
         XPathNodeIterator iterator; // enables node iteration

         try // get specified node from ComboBox
         {
            // select specified node
            iterator = xPath.Select( cboSelect.Text ); 
            DisplayIterator( iterator ); // print selection
         } // end try
         // catch invalid expressions
         catch ( System.Xml.XPath.XPathException argumentException )
         {
            MessageBox.Show( argumentException.Message, "Error",
               MessageBoxButtons.OK, MessageBoxIcon.Error );
         } // end catch
      } // end method btnSelect_Click

      // traverse to first child on btnFirstChild_Click event
      private void btnFirstChild_Click( object sender, EventArgs e )
      {
         TreeNode newTreeNode;

         // move to first child
         if ( xPath.MoveToFirstChild() )
         {
            newTreeNode = new TreeNode(); // create new node

            // set node's Text property to
            // either navigator's name or value
            DetermineType( newTreeNode, xPath );

            // add nodes to TreeNode node list
            tree.Nodes.Add( newTreeNode );
            tree = newTreeNode; // assign tree newTreeNode

            // update TreeView control
            treePath.ExpandAll(); // expand node in TreeView
            treePath.Refresh(); // force TreeView to update
            treePath.SelectedNode = tree; // highlight root
         } // end if
         else // node has no children
            MessageBox.Show( "Current Node has no children.",
               "", MessageBoxButtons.OK, MessageBoxIcon.Information );
      } // end method btnFirstChild_Click

      // traverse to node's parent on btnParent_Click event
      private void btnParent_Click( object sender, EventArgs e )
      {
         // move to parent
         if ( xPath.MoveToParent() )
         {
            tree = tree.Parent;

            // get number of child nodes, not including sub trees
            int count = tree.GetNodeCount( false );

            // remove all children
            for ( int i = 0; i < count; i++ )
               tree.Nodes.Remove( tree.FirstNode ); // remove child node

            // update TreeView control
            treePath.ExpandAll(); // expand node in TreeView
            treePath.Refresh(); // force TreeView to update
            treePath.SelectedNode = tree; // highlight root
         } // end if
         else // if node has no parent (root node)
            MessageBox.Show( "Current node has no parent.", "",
               MessageBoxButtons.OK, MessageBoxIcon.Information );
      } // end method btnParent_Click

      // find next sibling on btnNext_Click event
      private void btnNext_Click( object sender, EventArgs e )
      {
         // declare and initialize two TreeNodes
         TreeNode newTreeNode = null; 
         TreeNode newNode = null;

         // move to next sibling
         if ( xPath.MoveToNext() )
         {
            newTreeNode = tree.Parent; // get parent node

            newNode = new TreeNode(); // create new node

            // decide whether to display current node
            DetermineType( newNode, xPath ); 
            newTreeNode.Nodes.Add( newNode ); // add to parent node

            tree = newNode; // set current position for display

            // update TreeView control
            treePath.ExpandAll(); // expand node in TreeView
            treePath.Refresh(); // force TreeView to update
            treePath.SelectedNode = tree; // highlight root
         } // end if
         else // node has no additional siblings
            MessageBox.Show( "Current node is last sibling.", "",
               MessageBoxButtons.OK, MessageBoxIcon.Information );
      } // end method btnNext_Click

      // get previous sibling on btnPrevious_Click
      private void btnPrevious_Click( object sender, EventArgs e )
      {
         TreeNode parentTreeNode = null;
         
         // move to previous sibling
         if ( xPath.MoveToPrevious() )
         {
            parentTreeNode = tree.Parent; // get parent node
            parentTreeNode.Nodes.Remove( tree ); // delete current node
            tree = parentTreeNode.LastNode; // move to previous node

            // update TreeView control
            treePath.ExpandAll(); // expand tree node in TreeView
            treePath.Refresh(); // force TreeView to update
            treePath.SelectedNode = tree; // highlight root
         } // end if
         else // if current node has no previous siblings
            MessageBox.Show( "Current node is first sibling.", "",
               MessageBoxButtons.OK, MessageBoxIcon.Information );
      } // end method btnPrevious_Click

      // print values for XPathNodeIterator
      private void DisplayIterator( XPathNodeIterator iterator )
      {
         txtSelect.Clear();

         // prints selected node's values
         while ( iterator.MoveNext() )
            txtSelect.Text += iterator.Current.Value.Trim() + "\r\n";
      } // end method DisplayIterator

      // determine if TreeNode should display current node name or value
      private void DetermineType( TreeNode node, XPathNavigator xPath )
      {
         switch ( xPath.NodeType ) // determine NodeType
         {
            case XPathNodeType.Element: // if Element, get its name
               // get current node name, and remove whitespaces
               node.Text = xPath.Name.Trim();
               break;
            default: // obtain node values
               // get current node value and remove whitespaces
               node.Text = xPath.Value.Trim();
               break;
         } // end switch
      } // end method DetermineType
   } // end class PathNavigatorForm
} // end namespace PathNavigator

/**************************************************************************
 * (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
 * Pearson Education, Inc. All Rights Reserved.                           *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *************************************************************************/